import os
import configparser

from .utils     import get_datadir, CheckIfFileExists
from .version   import config_version

def create_config(IMAGES=12, IMAGE_QUALITY=80, FONT='', FONT_SIZE=30, CUSTOM_TEXT='', BG_COLOUR='white', FONT_COLOUR='black'):
    my_datadir = get_datadir() / "thumb-gen"

    try:
        my_datadir.mkdir(parents=True)

    except FileExistsError:
        pass

    finally:
        configfile_path = os.path.join(str(my_datadir), "config.ini")
        config_object = configparser.ConfigParser()
        config_object["DEFAULT"] = {"images": IMAGES, "image_quality": IMAGE_QUALITY, "font":FONT, "font_size":FONT_SIZE, "custom_text":CUSTOM_TEXT, "bg_colour":BG_COLOUR, "font_colour":FONT_COLOUR}
        config_object["VERSION"] = {"config_version": config_version}
        with open(configfile_path, 'w') as conf:
            config_object.write(conf)

        return True

def modify_config(options, value):
    my_datadir = get_datadir() / "thumb-gen"
    configfile_path = os.path.join(str(my_datadir), "config.ini")
    config_object  = configparser.ConfigParser()

    config_object.read(configfile_path)
    userinfo = config_object["DEFAULT"]

    if options == "images":
        userinfo["images"] = str(value)

    elif options == "image_quality":
        userinfo["image_quality"] = str(value)

    elif options == "font":
        userinfo["font"] = str(value)

    elif options == "font_size":
        userinfo["font_size"] = str(value)

    elif options == "custom_text":
        if value == '000' or value == 'clear':
            userinfo["custom_text"] = ''
        else:
            userinfo["custom_text"] = str(value)

    elif options == "bg_colour":
        userinfo["bg_colour"] = str(value)

    elif options == "font_colour":
        userinfo["font_colour"] = str(value)

    with open(configfile_path, 'w') as conf:
        config_object.write(conf)

    return True

def read_config(option):
    my_datadir = get_datadir() / "thumb-gen"
    configfile_path = os.path.join(str(my_datadir), "config.ini")
    config_object = configparser.ConfigParser()

    if CheckIfFileExists(configfile_path):
        config_object.read(configfile_path)

        try:
            config_object["VERSION"]["config_version"] == config_version
        except KeyError:
            create_config()

        config_object.read(configfile_path)
        if config_object["VERSION"]["config_version"] == config_version:
            default = config_object["DEFAULT"]

        else:
            create_config_return = create_config()
            if create_config_return:
                config_object.read(configfile_path)
                default = config_object["DEFAULT"]

    else:
        create_config_return = create_config()
        if create_config_return:
            config_object.read(configfile_path)
            default = config_object["DEFAULT"]

    loop = True
    while loop:
        try:
            if option == 'images':
                return int(default['images'])
            elif option == 'image_quality':
                return int(default['image_quality'])
            elif option == 'font':
                return str(default['font'])
            elif option == 'font_size':
                return int(default['font_size'])
            elif option == 'custom_text':
                return str(default['custom_text'])
            elif option == 'bg_colour':
                return str(default['bg_colour'])
            elif option == 'font_colour':
                return str(default['font_colour'])
            loop = False

        except KeyError:
            create_config_return = create_config()
            if create_config_return:
                config_object.read(configfile_path)
                default = config_object["DEFAULT"]



import os
import re
import sys
import datetime

from ffmpy      import FFmpeg
from PIL        import Image
from PIL        import ImageFont
from PIL        import ImageDraw
from PIL        import ImageColor

from .config    import read_config
from .utils     import listToString, video_info, get_file_size, convert_unit, packagePath

def font_info(text, font, font_size):
    try:
        font = ImageFont.truetype(font, font_size)
    except OSError:
        font = ImageFont.load_default()

    text_width = font.getsize(text)[0]
    text_height = font.getsize(text)[1]

    return text_width, text_height

def lining(text, font, font_size, image_width):
    lines = {'line1': []}
    tmp_list = []
    text_list = text.split(" ")
    tmp_list = text_list
    rounds = 0
    loop = True

    while loop:
        list_len = len(text_list)
        paragraph = listToString(tmp_list)
        text_width = font_info(paragraph, font, font_size)[0]

        rounds = rounds + 1

        if not text_width > image_width:
            loop2 = True
            list_number = 0

            while loop2:
                list_number = list_number + 1

                try:
                    tmp_line = lines['line{}'.format(list_number)]
                except KeyError:
                    tmp_line = []

                if '0tKJz' not in tmp_line:
                    lines['line{}'.format(list_number)] = [paragraph]
                    lines['line{}'.format(list_number)].append('0tKJz')

                    for i in range(0, (list_len - rounds) + 1):
                        try:
                            text_list.pop(0)
                        except IndexError:
                            pass

                    tmp_list = text_list
                    rounds = 0
                    loop2 = False

        else:
            tmp_list = tmp_list[:-1]

        if text_list == []:
            loop = False

    r_line = 0

    for line in lines:
        r_line = r_line + 1

        if "0tKJz" in lines['line{}'.format(r_line)]:
            lines['line{}'.format(r_line)].remove("0tKJz")

    return lines

def imageText(video_path, secure_tmp, bg_width, bg_height, custom_text, font_dir, font_size, bg_colour, font_colour):
    if font_dir == '':
        font_name = read_config('font')
        if font_name == '':
            package_dir = packagePath()
            font_name = os.path.join(package_dir, 'fonts', 'RobotoCondensed-Regular.ttf')

    else:
        font_name = font_dir

    if font_size == 0:
        font_size = read_config('font_size')

    if custom_text == 'True':
        custom_text = read_config('custom_text')
    elif custom_text == 'False':
        custom_text = ''

    if bg_colour == '':
        bg_colour = read_config('bg_colour')

    if font_colour == '':
        font_colour = read_config('font_colour')

    filename = re.split(pattern = r"[/\\]", string = video_path)
    filename = filename[-1]

    #file
    info_filename = "Filename: " + filename
    info_filesize = "Size: " + str(get_file_size(video_path)) + "MB"
    try:
        duration = round(float(video_info(video_path)[2]['duration']))
        info_duration = "Duration: " + str(datetime.timedelta(seconds=duration))
    except KeyError:
        info_duration = ''
    try:
        avg_bitrate = convert_unit(int(video_info(video_path)[2]['bit_rate']))
        info_avgbitrate = "avg. Bitrate: " + str(avg_bitrate) + "KB/s"
    except KeyError:
        info_avgbitrate = ''
    #video
    try:
        info_video = "Video: " + video_info(video_path)[0]['codec_name']
    except KeyError:
        info_video = ''
    try:
        info_video_res = str(video_info(video_path)[0]['width']) + 'x' + str(video_info(video_path)[0]['height'])
    except KeyError:
        info_video_res = ''
    try:
        video_bitrate = video_info(video_path)[0]['bit_rate']
        if str(video_bitrate) == 'N/A':
            raise KeyError
        info_video_bitrate = 'bitrate = ' + str(convert_unit(int(video_bitrate))) + "KB/s"
    except KeyError:
        info_video_bitrate = ''
    try:
        video_fps = video_info(video_path)[0]['avg_frame_rate'].split('/')
        video_fps = round(int(video_fps[0]) / int(video_fps[1]), 2)
        info_video_fps = str(video_fps) + 'fps'
    except KeyError:
        info_video_fps = ''
    #audio
    try:
        info_audio = "Audio: " + video_info(video_path)[1]['codec_name']
    except KeyError:
        info_audio = ''
    try:
        info_audio_rate = str(video_info(video_path)[1]['sample_rate']) + 'Hz'
    except KeyError:
        info_audio_rate = ''
    try:
        info_audio_channels = str(video_info(video_path)[1]['channels']) + ' channels'
    except KeyError:
        info_audio_channels = ''
    try:
        audio_bitrate = video_info(video_path)[1]['bit_rate']
        if str(audio_bitrate) == 'N/A':
            raise KeyError
        info_audio_bitrate = 'bitrate = ' + str(convert_unit(int(audio_bitrate))) + "KB/s"
    except KeyError:
        info_audio_bitrate = ''
    #custom
    custom_text_bx = custom_text

    info_line2 = info_filesize + '    ' + info_duration + '    ' + info_avgbitrate
    info_line3 = info_video + '    ' + info_video_res + '    ' + info_video_bitrate + '    ' + info_video_fps
    info_line4 = info_audio + '    ' + info_audio_rate + '    ' + info_audio_channels + '    ' + info_audio_bitrate

    font_height_filename = 0
    font_height_normal = font_info(info_duration, font_name, font_size)[1]
    font_height_custom_text = 0

    filename_text_lines = lining(info_filename, font_name, font_size, bg_width)
    rounds = 0
    for items in filename_text_lines:
        rounds = rounds + 1
        for lines in filename_text_lines['line{}'.format(rounds)]:
            if lines != []:
                font_height = font_info(lines, font_name, font_size)[1]
                font_height_filename = font_height_filename + font_height + 5

    rounds = 0
    if not custom_text_bx == '':
        custom_text_lines = lining(custom_text_bx, font_name, font_size, bg_width)
        for items in custom_text_lines:
            rounds = rounds + 1
            for lines in custom_text_lines['line{}'.format(rounds)]:
                if lines != []:
                    font_height = font_info(lines, font_name, font_size)[1]
                    font_height_custom_text = font_height_custom_text + font_height

    text_area_height = 5 + font_height_filename + (font_height_normal + 5) * 3 + font_height_custom_text

    bg_new_height = text_area_height + bg_height

    try:
        bg_colour = ImageColor.getrgb(bg_colour)
    except ValueError:
        bg_colour = bg_colour
        print("ValueError: unknown color specifier: {}".format(bg_colour))
        print("This can be fixed by changing the configurations.\n" \
              "Ex: Background colour = 'white'  /  Background colour = '#ffffff'")
        sys.exit()

    try:
        font_colour = ImageColor.getrgb(font_colour)
    except ValueError:
        font_colour = font_colour
        print("ValueError: unknown color specifier: {}".format(font_colour))
        print("This can be fixed by changing the configurations.\n" \
              "Ex: Font colour = 'black'  / Font colour = '#ffffff'")
        sys.exit()

    img = Image.new('RGB', (bg_width, bg_new_height), bg_colour)
    img.save(os.path.join(secure_tmp, 'bg.png'))

    background = Image.open(os.path.join(secure_tmp, 'bg.png'))
    org_width = background.size[0]

    draw = ImageDraw.Draw(background)

    try:
        font = ImageFont.truetype(font_name, font_size)
    except OSError:
        print("{} file not found! Default font is loaded.".format(font_name))
        package_dir = packagePath()
        font_name = os.path.join(package_dir, 'fonts', 'RobotoCondensed-Regular.ttf')
        font = ImageFont.truetype(font_name, font_size)

    x = 10
    y = 5

    rounds = 0

    #line1
    info_filename_line = lining(info_filename, font_name, font_size, bg_width)
    for items in info_filename_line:
        rounds = rounds + 1
        for lines in info_filename_line['line{}'.format(rounds)]:
            if lines != []:
                font_height = font_info(info_filename, font_name, font_size)[1]
                draw.text((x, y), lines, font_colour, font=font)
                y = y + font_height

    font_height = font_info(info_filesize, font_name, font_size)[1]

    #line2
    draw.text((x, y), info_line2, font_colour, font=font)
    y = y + 5 + font_height

    #line3
    draw.text((x, y), info_line3, font_colour, font=font)
    y = y + 5 + font_height

    #line4
    draw.text((x, y), info_line4, font_colour, font=font)
    y = y + 5 + font_height

    rounds = 0
    if not custom_text == '':
        text_lines = lining(custom_text, font_name, font_size, org_width)
        for items in text_lines:
            rounds = rounds + 1
            for lines in text_lines['line{}'.format(rounds)]:
                if lines != []:
                    font_height = font_info(lines, font_name, font_size)[1]
                    draw.text((x, y), lines, font_colour, font=font)
                    y = y + font_height
    y = y + 5

    background.save(os.path.join(secure_tmp, 'bg.png'))

    return y + 5

def screenshots(video_path, screenshot_folder):
    for img in os.listdir(screenshot_folder):
        os.remove(os.path.join(screenshot_folder, img))

    tm_video_path = re.split(pattern = r"[/\\]", string = video_path)
    video_duration = int(round(float(video_info(video_path)[2]['duration']), 2))
    frame_time = round((video_duration / read_config('images')), 2)

    screenshot_path = os.path.join(screenshot_folder, tm_video_path[-1])
    event = FFmpeg(inputs={video_path: None}, \
            outputs={screenshot_path + '_screen%d.png': \
            ['-hide_banner', '-nostats', '-loglevel', '0', '-vf', 'fps=1/' + str(frame_time)]})
    event.run()

    return True

def resize(screenshot_folder, resize_folder):
    for img in os.listdir(resize_folder):
        os.remove(resize_folder + img)

    for img in os.listdir(screenshot_folder):
        image = Image.open(os.path.join(screenshot_folder, img))
        org_width, org_height = image.size

        new_height = 300 * org_height / org_width

        resized_im = image.resize((300, round(new_height)))
        resized_im.save(os.path.join(resize_folder, 'resized_') + img)

    return True

def thumb(video_path, output_folder, resize_folder, secure_temp, custom_text, font_dir, font_size, bg_colour, font_colour):
    for img in os.listdir(resize_folder):
        image = Image.open(os.path.join(resize_folder, img))
        r_new_width, new_height = image.size
        break

    img_rows = read_config('images') / 3
    tmp_var = str(img_rows).split('.')
    if tmp_var[1] != '0':
        img_rows = int(img_rows) + 1
    img_rows = int(img_rows)

    bg_new_width = int((r_new_width * 3) + 20)
    bg_new_height = int((new_height * img_rows) + ((5 * img_rows) + 5))

    y = imageText(video_path, secure_temp, bg_new_width, bg_new_height, custom_text, font_dir, font_size, bg_colour, font_colour)

    backgroud = Image.open(os.path.join(secure_temp, 'bg.png'))

    img_list = []
    for img in os.listdir(resize_folder):
        image = Image.open(os.path.join(resize_folder, img))
        img_list.append(image)

    back_im = backgroud.copy()

    count = 0
    x = 5
    for img in img_list:
        count =  count + 1
        if (count - 1) % 3 == 0 and count != 1:
            y = y + new_height + 5
            x = 5

        back_im.paste(img_list[count - 1], (x, y))
        x = x + r_new_width + 5

    back_im.save(output_folder + '.png', quality=read_config('image_quality'))

    return True



import  sys
import  shutil

from    .options        import  parseOpts, begin
from    .worker         import  Generator
from    .viewer         import  helps, args_error

def run(argument_list = False):
    try:
        args, opt = parseOpts(argument_list)
    except TypeError:
        args_error()
        sys.exit()
    videos = begin(args, opt)

    for video_path in videos:
        app = Generator(video_path)
        app.run()

    sys.exit()



import os
import re
import tempfile

from .application   import screenshots, resize, thumb
from .viewer        import print_process, print_success
from .utils         import listToString

class Generator:
    def __init__(self, video_path, output_path='', custom_text='True', font_dir='', font_size=0, bg_colour='', font_colour=''):
        self.video_path = video_path

        if output_path == '':
            self.output_path = self.video_path[:-4]
            self.output_folder = listToString(re.split(pattern = r"[/\\]", string = self.video_path)[:-1], "sys")

        else:
            self.filename = re.split(pattern = r"[/\\]", string = self.video_path)[-1]
            self.output_path = os.path.join(output_path, self.filename[:-4])
            self.output_folder = self.output_path

        self.custom_text = str(custom_text)
        self.font_dir = font_dir

        if isinstance(font_size, int):
            self.font_size = font_size
        elif isinstance(font_size, str):
            raise ValueError("Font size must be an integer")

        self.bg_colour = bg_colour
        self.font_colour = font_colour

        self.temp_dir = tempfile.TemporaryDirectory()
        self.secure_temp = self.temp_dir.name
        self.screenshot_folder = os.path.join(self.secure_temp, 'screenshots')
        self.resize_folder = os.path.join(self.secure_temp, 'resized')
        os.mkdir(self.screenshot_folder)
        os.mkdir(self.resize_folder)

    def run(self):
        print_process(self.video_path)
        screenshots(self.video_path, self.screenshot_folder)
        resize(self.screenshot_folder, self.resize_folder)
        thumb_out = thumb(self.video_path, self.output_path, self.resize_folder, self.secure_temp, self.custom_text, self.font_dir, self.font_size, self.bg_colour, self.font_colour)

        if thumb_out:
            print_success(self.output_folder)

        return 1



import  os
import  sys
import  getopt

from .viewer        import args_error, helps, deco
from .version       import __version__
from .viewer        import configurations
from .config        import modify_config
from .utils         import CheckIfFileExists

videos = []

def parseOpts(argument_list):
    deco()

    try:
        opts, args = getopt.getopt(argument_list,"cvhf:d:",["config","version","help","file=","dir="])

    except getopt.GetoptError:
        args_error(argument_list)
        sys.exit()

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            helps()
            sys.exit()

        elif opt in("-f", "--file"):
            if arg == '':
                args_error()
                sys.exit()

            else:
                return arg, opt

        elif opt in ("-d", "--dir"):
            return arg, opt

        elif opt in ("-v", "--version"):
            print(__version__)
            sys.exit()

        elif opt in ("-c", "--config"):
            conf_images, conf_image_quality, conf_font, conf_font_size, conf_custom_text, conf_bg_colour, conf_font_colour = configurations()
            if conf_images != 0:
                modify_config('images', conf_images)

            if conf_image_quality != 0:
                modify_config('image_quality', conf_image_quality)

            if conf_font != '0':
                modify_config('font', conf_font)

            if conf_font_size != 0:
                modify_config('font_size', conf_font_size)

            if conf_custom_text != '':
                modify_config('custom_text', conf_custom_text)

            if conf_bg_colour != '':
                modify_config('bg_colour', conf_bg_colour)

            if conf_font_colour != '':
                modify_config('font_colour', conf_font_colour)

            sys.exit()

def begin(args='', opt=''):
    current_folder = os.getcwd()

    if opt in ('-d', '--dir') and args != '':
        if CheckIfFileExists(args):
            for video in os.listdir(args):
                if video.endswith('.mp4') or video.endswith('.mkv'):
                    videos.append(os.path.join(args, video))

        elif CheckIfFileExists(os.path.join(current_folder, args)):
            for video in os.listdir(os.path.join(current_folder, args)):
                if video.endswith('.mp4') or video.endswith('.mkv'):
                    videos.append(os.path.join(current_folder, args, video))

        else:
            print("Folder not found!")
            sys.exit()

    elif opt in ('-f', '--file') and args != '':
        if CheckIfFileExists(args):
            videos.append(args)

        elif CheckIfFileExists(os.path.join(current_folder, args)):
            videos.append(os.path.join(current_folder, args))

        else:
            print("File not found!")
            sys.exit()

    return videos



import  pathlib

from    setuptools      import setup, find_packages

exec(open("thumb_gen/version.py").read())

here = pathlib.Path(__file__).parent
with open('README.rst') as readme_file:
    long_description = readme_file.read()

setup(
    name="thumb_gen",
    version=__version__,
    description="Python application that can be used to generate video thumbnail for mp4 and mkv file types.",
    long_description=long_description,
    author="tharindu.dev",
    author_email="tharindu.nm@yahoo.com",
    url="https://github.com/truethari/thumb-gen",
    keywords="thumbnails video screenshot",
    license='MIT',
    project_urls={
        "Bug Tracker": "https://github.com/truethari/thumb-gen/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    packages=['thumb_gen'],
    include_package_data=True,
    package_data = {'' : ['fonts/*.ttf']},
    install_requires=["Pillow", "infomedia", "ffmpy"],
    entry_points={
        "console_scripts": [
            "thumb-gen=thumb_gen.__main__:run",
        ]
    },
)



import os
import sys
import pathlib

from infomedia  import mediainfo

import thumb_gen as _

def get_datadir() -> pathlib.Path:

    """
    Returns a parent directory path
    where persistent application data can be stored.

    # linux: ~/.local/share
    # macOS: ~/Library/Application Support
    # windows: C:/Users/<USER>/AppData/Roaming
    """

    home = pathlib.Path.home()

    if sys.platform == "win32":
        return home / "AppData/Roaming"
    elif sys.platform == "linux":
        return home / ".local/share"
    elif sys.platform == "darwin":
        return home / "Library/Application Support"

def check_os():
    if sys.platform == 'win32':
        return 'win32'
    elif sys.platform == 'linux':
        return 'linux'
    elif sys.platform == 'darwin':
        return 'darwin'

def listToString(s, chars=" "):
    str1 = chars
    if chars == 'sys':
        oss = check_os()
        if oss == 'win32':
            return ('\\'.join(s))
        else:
            return ('/'.join(s))
    else:
        return (str1.join(s))

def video_info(video_path):
    audio_properties = {}
    video_properties = {}
    media_info = mediainfo(video_path)
    for key in media_info:
        try:
            if media_info[key]['codec_type'] == 'audio':
                audio_properties = media_info[key]
            elif media_info[key]['codec_type'] == 'video':
                video_properties = media_info[key]
        except KeyError:
            continue

    return video_properties, audio_properties, media_info['format']

def convert_unit(size_in_bytes, unit='KB'):
    if unit == "KB":
        return round(size_in_bytes/1024, 2)
    elif unit == "MB":
        return round(size_in_bytes/(1024*1024), 2)
    elif unit == "GB":
        return round(size_in_bytes/(1024*1024*1024), 2)
    else:
        return size_in_bytes

def get_file_size(file_name, unit="MB"):
    size = os.path.getsize(file_name)
    return convert_unit(size, unit)

def CheckIfFileExists(file_path):
    if os.path.exists(file_path):
        return True
    else:
        return False

def packagePath():
    return _.__path__[0]



import sys

from .config import read_config
from .utils import CheckIfFileExists, check_os
from datetime import datetime
from PIL import ImageColor

now = datetime.now()
current_time = now.strftime("%H:%M:%S")

def deco():
        print("""
  _____ _                     _                            
 /__   \ |__  _   _ _ __ ___ | |__         __ _  ___ _ __  
   / /\/ '_ \| | | | '_ ` _ \| '_ \ _____ / _` |/ _ \ '_ \ 
  / /  | | | | |_| | | | | | | |_) |_____| (_| |  __/ | | |
  \/   |_| |_|\__,_|_| |_| |_|_.__/       \__, |\___|_| |_|
                                          |___/            
                """)

def args_error(argument_list = False):
        if argument_list == False:
                print("Usage: thumb-gen [options] path/file\n\nerror: You must provide at least one option.")
        else:
                print("ERROR: unknown command {}".format(argument_list))

def helps():
        print ("""
    -h, --help          show this help message and exit
    -c, --config        change screenshots for thumbnail image and image quality
    -v, --version       show program's version number and exit

 Options:
    -f, --file          input a single video from current directory
    -d, --dir           input videos from a directory
    """)

def configurations():
        print("Configurations\n")
        print("Images = {}\n" \
              "Thumbnail Quality = {}\n" \
              "Font = {}\n" \
              "Font Size = {}\n" \
              "Custom Text = {}\n" \
              "Background Colour = {}\n" \
              "Font Colour = {}\n" \
              .format(read_config('images'), \
                      read_config('image_quality'), \
                      read_config('font'), \
                      read_config('font_size'), \
                      read_config('custom_text'), \
                      read_config('bg_colour'), \
                      read_config('font_colour')))

        while True:
                try:
                        while True:
                                print("If you do not want to change the values, leave the input blank and press Enter.\n")
                                print("CTRL + C to exit.")
                                try:
                                        images = int(input("Images: ") or 0)
                                        break
                                except ValueError:
                                        print("Invalid input! Please enter a valid number.")

                        while True:
                                try:
                                        while True:
                                                image_quality = int(input("Thumbnail Quality (10 - 100): ") or 0)
                                                if 101 > image_quality > 9 or image_quality == 0:
                                                        break
                                                else:
                                                        print("Enter number between 10 - 100!")
                                        break
                                except ValueError:
                                        print("Invalid input! Please enter a valid number.")

                        while True:
                                font_path = str(input("Font Path: ") or '0')
                                font_path_status = CheckIfFileExists(font_path)
                                if font_path_status or font_path == '0':
                                        break
                                else:
                                        print("No font file found. Check the path and re-enter it")

                        while True:
                                try:
                                        while True:
                                                font_size = int(input("Font Size (10 - 100): ") or 0)
                                                if 9 < font_size < 101 or font_size == 0:
                                                        break
                                                else:
                                                        print("Enter number between 10 - 100!")
                                        break
                                except ValueError:
                                        print("Invalid input! Please enter a valid number.")

                        print("Input 'clear' or '000' to clear custom text")
                        custom_text = str(input("Custom text: ") or '')

                        while True:
                                try:
                                        bg_colour = str(input("Background Colour: ") or '')
                                        if bg_colour == '':
                                                pass
                                        else:
                                                ImageColor.getrgb(bg_colour)
                                except ValueError:
                                        print("Color not recognized: {}. Enter a valid colour!".format(bg_colour))
                                else:
                                        break

                        while True:
                                try:
                                        font_colour = str(input("Font Colour: ") or '')
                                        if font_colour == '':
                                                pass
                                        else:
                                                ImageColor.getrgb(font_colour)
                                except ValueError:
                                        print("Color not recognized: {}. Enter a valid colour!".format(font_colour))
                                else:
                                        break

                except KeyboardInterrupt:
                        sys.exit('\n')

                break

        return images, image_quality, font_path, font_size, custom_text, bg_colour, font_colour

def print_process(name):
        oss = check_os()
        if oss == 'linux':
                print("\033[33m [{}]  \033[36m Processing: {}\033[00m" .format(current_time, name))
        else:
                print(" [{}]   Processing: {}".format(current_time, name))

def print_success(name):
    oss = check_os()
    if oss == 'linux':
        print("\033[33m [{}]  \033[36m Thumbnail saved in: {}\033[00m" .format(current_time, name))
    else:
        print(" [{}]   Thumbnail saved in: {}".format(current_time, name))
